/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton.impl;

import java.util.List;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Collections;

import org.xmloperator.java.skeleton.model.ClassOrInterface;
import org.xmloperator.java.skeleton.model.Package;
import org.xmloperator.java.skeleton.model.IdentifiedPackage;
import org.xmloperator.java.skeleton.model.Skeleton;
import org.xmloperator.java.skeleton.impl.PackageImpl;
import org.xmloperator.java.skeleton.impl.IdentifiedPackageImpl;

public final class SkeletonImpl implements Skeleton {
  private final PackageImpl defaultPackage = new PackageImpl();
  private final List primaryPackageList = new ArrayList();

  public SkeletonImpl() {
  }

  //////////////////////////
  // Skeleton implementation
  //////////////////////////

  public Package getDefaultPackage() {
    return this.defaultPackage;
  }

  public int getPrimaryPackageCount() {
    return this.primaryPackageList.size();
  }

  public IdentifiedPackage getPrimaryPackage(int index) {
    return (IdentifiedPackage)this.primaryPackageList.get(index);
  }

  public boolean addPrimaryPackage(IdentifiedPackage primaryPackage) {
    if (primaryPackage.getParent() != null)
      throw new IllegalArgumentException();
    if (this.primaryPackageList.contains(primaryPackage))
      return false;
    this.primaryPackageList.add(primaryPackage);
    return true;
  }

  public IdentifiedPackage getPrimaryPackage(String identifier) {
    for (int k = 0; k < this.primaryPackageList.size(); k++) {
      IdentifiedPackage primaryPackage =
          (IdentifiedPackage)this.primaryPackageList.get(k);
      if (primaryPackage.getIdentifier().equals(identifier))
        return primaryPackage;
    }
    return null;
  }

  public IdentifiedPackage addPrimaryPackage(String identifier) {
    IdentifiedPackage primaryPackage = this.getPrimaryPackage(identifier);
    if (primaryPackage != null)
      return primaryPackage;
    primaryPackage = new IdentifiedPackageImpl(null, identifier);
    this.primaryPackageList.add(primaryPackage);
    return primaryPackage;
  }

  public IdentifiedPackage getPackage(String qualifiedIdentifier) {
    int dotPos = qualifiedIdentifier.indexOf('.');
    String primaryIdentifier = dotPos < 0 ? qualifiedIdentifier :
        qualifiedIdentifier.substring(0, dotPos);
    IdentifiedPackage primaryPackage =
        this.getPrimaryPackage(primaryIdentifier);
    return dotPos < 0 || primaryPackage == null ? primaryPackage :
        primaryPackage.getDescendantPackage(
        qualifiedIdentifier.substring(dotPos + 1));
  }

  public IdentifiedPackage addPackage(String qualifiedIdentifier) {
    int dotPos = qualifiedIdentifier.indexOf('.');
    String childIdentifier = dotPos < 0 ? qualifiedIdentifier :
        qualifiedIdentifier.substring(0, dotPos);
    IdentifiedPackage primaryPackage = this.addPrimaryPackage(childIdentifier);
    return dotPos < 0 ? primaryPackage :
        primaryPackage.addDescendantPackage(
        qualifiedIdentifier.substring(dotPos + 1));
  }

  public ClassOrInterface getClassOrInterface(String qualifiedIdentifier) {
    int dotPos = qualifiedIdentifier.lastIndexOf('.');
    String classOrInterfaceIdentifier;
    Package parentPackage;
    if (dotPos < 0) {
      classOrInterfaceIdentifier = qualifiedIdentifier;
      parentPackage = this.defaultPackage;
    }
    else {
      classOrInterfaceIdentifier = qualifiedIdentifier.substring(dotPos + 1);
      parentPackage = this.getPackage(qualifiedIdentifier.substring(0, dotPos));
    }
    return parentPackage != null ?
        parentPackage.getClassOrInterface(classOrInterfaceIdentifier) : null;
  }

  public ClassOrInterface addClassOrInterface(String qualifiedIdentifier) {
    int dotPos = qualifiedIdentifier.lastIndexOf('.');
    String classOrInterfaceIdentifier;
    Package parentPackage;
    if (dotPos < 0) {
      classOrInterfaceIdentifier = qualifiedIdentifier;
      parentPackage = this.defaultPackage;
    }
    else {
      classOrInterfaceIdentifier = qualifiedIdentifier.substring(dotPos + 1);
      parentPackage = this.addPackage(qualifiedIdentifier.substring(0, dotPos));
    }
    return parentPackage.addClassOrInterface(classOrInterfaceIdentifier);
  }

  public void sort() {
    this.defaultPackage.sortClassesAndInterfaces();
    this.sortPrimaryPackages();
    for (int k = 0; k < this.primaryPackageList.size(); k++)
      ((IdentifiedPackage)this.primaryPackageList.get(k)).sort();
  }

  private void sortPrimaryPackages() {
    Collections.sort(this.primaryPackageList,
                     new Comparator() {
      public int compare(Object o1, Object o2) {
        return ((IdentifiedPackage)o1).getIdentifier().compareTo(
            ( (IdentifiedPackage)o2).getIdentifier());
      }
    });
  }
}
