/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton.util;

import java.io.IOException;
import java.io.InputStream;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

import org.xmloperator.java.skeleton.model.ClassOrInterface;
import org.xmloperator.java.skeleton.model.Package;
import org.xmloperator.java.skeleton.model.IdentifiedPackage;
import org.xmloperator.java.skeleton.model.Skeleton;
import org.xmloperator.java.skeleton.util.InvalidSkeletonException;

/**
 * Build a Skeleton object by parsing its XML file.
 */
public class SkeletonBuilderFromSAX extends DefaultHandler {

  private static final String JAVASKELETON_NAMESPACE_URI =
      "http://www.xmloperator.net/namespace/java";

  private static final String WARNING_REDEFINED_CLASS_OR_INTERFACE =
      "Redefined class or interface";

  private static final String ERROR_USE_ELEMENT_OUT_OF_CONTEXT =
      "\"use\" element out of context";
  private static final String ERROR_CLASSORINTERFACE_ELEMENT_OUT_OF_CONTEXT =
      "\"classOrInterface\" element out of context";
  private static final String ERROR_PACKAGE_ELEMENT_OUT_OF_CONTEXT =
      "\"package\" element out of context";
  private static final String ERROR_DEFAULTPACKAGE_ELEMENT_OUT_OF_CONTEXT =
      "\"defaultPackage\" element out of context";
  private static final String ERROR_IMPORT_ELEMENT_OUT_OF_CONTEXT =
      "\"import\" element out of context";
  private static final String ERROR_JAVASKELETON_ELEMENT_OUT_OF_CONTEXT =
      "\"javaSkeleton\" element out of context";
  private static final String ERROR_IDENTIFIER_ATTRIBUTE_IS_MISSING =
      "\"identifier\" attribute is missing";
  private static final String ERROR_QUALIFIED_IDENTIFIER_ATTRIBUTE_IS_MISSING =
      "\"qualifiedIdentifier\" attribute is missing";
  private static final String ERROR_UNKNOWN_ELEMENT = "Unknown element";

  private Skeleton skeleton;
  private String pathName;
  private boolean printWarning;
  private boolean isImport = false;
  private Package currentPackage = null;
  private ClassOrInterface currentClassOrInterface = null;
  private boolean intoUse = false;
  private int otherElementStackLength = 0;
  private int startAndEndElementCount = 0;

  public static final void buildSkeleton(
      Skeleton skeleton, String pathName, boolean printWarning,
      InputStream inputStream)
      throws IOException, SAXException, ParserConfigurationException {
    SkeletonBuilderFromSAX skeletonBuilder =
        new SkeletonBuilderFromSAX(skeleton, printWarning);
    skeletonBuilder.setPathName(pathName);
    skeletonBuilder.buildSkeleton(inputStream);
  }

  private SkeletonBuilderFromSAX(Skeleton skeleton, boolean printWarning) {
    if (skeleton == null)
      throw new IllegalArgumentException();
    this.skeleton = skeleton;
    this.printWarning = printWarning;
  }

  private void setPathName(String pathName) {
    this.pathName = pathName;
  }

  private final void buildSkeleton(InputStream inputStream)
      throws IOException, SAXException, ParserConfigurationException {
    SAXParserFactory factory = SAXParserFactory.newInstance();
    factory.setNamespaceAware(true);
    SAXParser parser = factory.newSAXParser();
    parser.parse(inputStream, (DefaultHandler)this);
  }

  private void warning(String message) {
    System.out.println(this.extendedMessage(message));
  }

  private void error(String message) {
    throw new InvalidSkeletonException(this.extendedMessage(message));
  }

  private String extendedMessage(String message) {
    return message + " in " + this.pathName + " at "
        + this.startAndEndElementCount + " start and end element count";
  }

  ////////////////////////////////
  // ContentHandler implementation
  ////////////////////////////////

  public void startElement(String uri, String localName, String qName,
                           Attributes attributes) throws SAXException {
    this.startAndEndElementCount++;
    if (this.otherElementStackLength > 0
        || !uri.equals(JAVASKELETON_NAMESPACE_URI)) {
      this.otherElementStackLength++;
    }
    else if (localName.equals("type")) {
      if (this.intoUse) {
        String qualifiedIdentifier = attributes.getValue("qualifiedIdentifier");
        if (qualifiedIdentifier == null)
          error(ERROR_QUALIFIED_IDENTIFIER_ATTRIBUTE_IS_MISSING);
        ClassOrInterface classOrInterface =
            this.skeleton.addClassOrInterface(qualifiedIdentifier);
        this.currentClassOrInterface.getUsedClassAndInterfaceList().
            addClassOrInterface(classOrInterface);
        classOrInterface.getUserClassAndInterfaceList().
            addClassOrInterface(this.currentClassOrInterface);
      }
    }
    else if (localName.equals("uses")) {
      if (this.isImport || this.intoUse || this.currentClassOrInterface == null
          || this.currentClassOrInterface.isDefined())
        error(ERROR_USE_ELEMENT_OUT_OF_CONTEXT);
      this.currentClassOrInterface.makeDefined();
      this.intoUse = true;
    }
    else if (localName.equals("usedBy")) {
    }
    else if (localName.equals("classOrInterface")) {
      if (this.currentPackage == null
          || this.currentClassOrInterface != null)
        error(ERROR_CLASSORINTERFACE_ELEMENT_OUT_OF_CONTEXT);
      String identifier = attributes.getValue("identifier");
      if (identifier == null)
        error(ERROR_IDENTIFIER_ATTRIBUTE_IS_MISSING);
      if (printWarning) {
        ClassOrInterface classOrInterface =
            this.currentPackage.getClassOrInterface(identifier);
        if (classOrInterface != null && classOrInterface.isDefined())
          warning(WARNING_REDEFINED_CLASS_OR_INTERFACE + " : "
              + classOrInterface.getQualifiedIdentifier());
      }
      this.currentClassOrInterface =
          this.currentPackage.addClassOrInterface(identifier);
    }
    else if (localName.equals("package")) {
      if (this.currentClassOrInterface != null
          || (this.currentPackage != null && this.currentPackage.isDefault()))
        error(ERROR_PACKAGE_ELEMENT_OUT_OF_CONTEXT);
      String identifier = attributes.getValue("identifier");
      if (identifier == null)
        error(ERROR_IDENTIFIER_ATTRIBUTE_IS_MISSING);
      this.currentPackage = this.currentPackage != null ?
          ((IdentifiedPackage)this.currentPackage).addChildPackage(identifier) :
          this.skeleton.addPrimaryPackage(identifier);
    }
    else if (localName.equals("defaultPackage")) {
      if (this.currentPackage != null)
        error(ERROR_DEFAULTPACKAGE_ELEMENT_OUT_OF_CONTEXT);
      this.currentPackage = this.skeleton.getDefaultPackage();
    }
    else if (localName.equals("import")) {
      if (this.currentPackage != null || isImport)
        error(ERROR_IMPORT_ELEMENT_OUT_OF_CONTEXT);
      this.isImport = true;
    }
    else if (localName.equals("javaSkeleton")) {
      if (this.currentPackage != null)
        error(ERROR_JAVASKELETON_ELEMENT_OUT_OF_CONTEXT);
    }
    else
      error(ERROR_UNKNOWN_ELEMENT + " : " + localName);
  }

  public void endElement(String uri, String localName, String qName)
      throws SAXException {
    this.startAndEndElementCount++;
    if (this.otherElementStackLength > 0)
      this.otherElementStackLength--;
    else if (localName.equals("uses"))
      this.intoUse = false;
    else if (localName.equals("classOrInterface"))
      this.currentClassOrInterface = null;
    else if (localName.equals("package"))
      this.currentPackage =
          ((IdentifiedPackage)this.currentPackage).getParent();
    else if (localName.equals("defaultPackage"))
      this.currentPackage = null;
    else if (localName.equals("import"))
      this.isImport = false;
  }
}
