/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton.util;

import java.util.List;

import org.xmloperator.java.syntax.CompilationUnitHandler;
import org.xmloperator.java.skeleton.model.ClassOrInterface;
import org.xmloperator.java.skeleton.model.Package;
import org.xmloperator.java.skeleton.model.Skeleton;
import org.xmloperator.java.skeleton.util.InvalidSkeletonException;

/**
 * Build a Skeleton object by parsing a compilation unit.
 */
public class SkeletonBuilderFromSource implements CompilationUnitHandler {
  private static final boolean DEBUG = false;
  private static final String ERROR_DUPLICATE_CLASS_OR_INTERFACE_IDENTIFIER =
      "Duplicate class or interface identifier";
  private Skeleton skeleton;
  private boolean forImport;
  private Package currentPackage = null;
  private ClassOrInterface currentClassOrInterface = null;

  public SkeletonBuilderFromSource(Skeleton skeleton, boolean forImport) {
    if (skeleton == null)
      throw new IllegalArgumentException();
    this.skeleton = skeleton;
    this.forImport = forImport;
  }

  private void addClassOrInterface(String classOrInterfaceIdentifier) {
    if (this.currentClassOrInterface != null)
      throw new IllegalStateException();
    ClassOrInterface classOrInterface =
        this.currentPackage.addClassOrInterface(classOrInterfaceIdentifier);
    if (classOrInterface.isDefined())
      throw new InvalidSkeletonException(
          ERROR_DUPLICATE_CLASS_OR_INTERFACE_IDENTIFIER + " : "
          + classOrInterface.getQualifiedIdentifier());
    if (!this.forImport)
      classOrInterface.makeDefined();
    this.currentClassOrInterface = classOrInterface;
  }

  private void addUsage(String classOrInterfaceQualifiedIdentifier) {
    if (!this.forImport) {
      ClassOrInterface classOrInterface =
          this.skeleton.addClassOrInterface(classOrInterfaceQualifiedIdentifier);
      this.currentClassOrInterface.getUsedClassAndInterfaceList().
          addClassOrInterface(classOrInterface);
      classOrInterface.getUserClassAndInterfaceList().
          addClassOrInterface(this.currentClassOrInterface);
    }
  }

  private void addUsage(List classOrInterfaceQualifiedIdentifierList) {
    for (int k = 0; k < classOrInterfaceQualifiedIdentifierList.size(); k++)
      this.addUsage((String)classOrInterfaceQualifiedIdentifierList.get(k));
  }

  ////////////////////////////////////////
  // CompilationUnitHandler implementation
  ////////////////////////////////////////

  public void startCompilationUnit(String packageQualifiedIdentifier) {
    if (this.currentPackage != null)
      throw new IllegalStateException();
    this.currentPackage = packageQualifiedIdentifier == null ?
        skeleton.getDefaultPackage() :
        skeleton.addPackage(packageQualifiedIdentifier);
  }

  public void endCompilationUnit() {
    if (this.currentPackage == null)
      throw new IllegalStateException();
    this.currentPackage = null;
  }

  public void startClass(String classIdentifier,
                         String extendedClassQualifiedIdentifier,
                         List implementedInterfaceQualifiedIdentifierList) {
    this.addClassOrInterface(classIdentifier);
    if (DEBUG)
      System.out.println(
          "startClass : "
          + this.currentClassOrInterface.getQualifiedIdentifier());
    if (extendedClassQualifiedIdentifier != null) {
      if (DEBUG)
        System.out.println("extends : " + extendedClassQualifiedIdentifier);
      this.addUsage(extendedClassQualifiedIdentifier);
    }
    if (DEBUG)
      for (int k = 0; k < implementedInterfaceQualifiedIdentifierList.size();
           k++)
        System.out.println(
            "implements : "
            + (String)implementedInterfaceQualifiedIdentifierList.get(k));
    this.addUsage(implementedInterfaceQualifiedIdentifierList);
  }

  public void startInterface(String interfaceIdentifier,
                             List extendedInterfaceQualifiedIdentifierList) {
    this.addClassOrInterface(interfaceIdentifier);
    if (DEBUG) {
      System.out.println(
          "startInterface : "
          + this.currentClassOrInterface.getQualifiedIdentifier());
      for (int k = 0; k < extendedInterfaceQualifiedIdentifierList.size(); k++)
        System.out.println(
            "extends : "
            + (String) extendedInterfaceQualifiedIdentifierList.get(k));
    }
    this.addUsage(extendedInterfaceQualifiedIdentifierList);
  }

  public void endClassOrInterface() {
    if (DEBUG)
      System.out.println(
          "endClassOrInterface : "
          + this.currentClassOrInterface.getQualifiedIdentifier());
    if (this.currentClassOrInterface == null)
      throw new IllegalStateException();
    this.currentClassOrInterface = null;
  }

  public void classOrInterfaceReference(
      String classOrInterfaceQualifiedIdentifier) {
    if (DEBUG)
      System.out.println("references : " + classOrInterfaceQualifiedIdentifier);
    this.addUsage(classOrInterfaceQualifiedIdentifier);
  }

  public void instanciation(String classQualifiedIdentifier) {
    if (DEBUG)
      System.out.println("instanciates : " + classQualifiedIdentifier);
    this.addUsage(classQualifiedIdentifier);
  }
}
